/*
 * extern.cpp
 *
 * Copyright 2011-2012. All rights reserved.
 */


#include <cstdlib>
#include <vector>
#include <algorithm>
#include "extern.h"

using namespace std;

extern const float ALPHA;
extern const float BETA;
extern const float LAMBDA;
extern const float GAMMA;
extern const float DELTA;
extern const float ORTH_COEF;
extern const float GRAPHLET_COEF;


int GetFinalSimMatrix(float**           finalSimMatrix,
                      SimilarityMatrix& simMatrix,
                      float**           simGraphletMatrix,
                      float             coefSimGraphlet)
{
    for (int i = 0; i < simMatrix.nHNode; i++)
    {
        for (int j = 0; j < simMatrix.nGNode; j++)
        {
            finalSimMatrix[i][j] = (1-coefSimGraphlet) * simMatrix.matrix[i][j] + coefSimGraphlet * simGraphletMatrix[i][j];
        }
    }
    return 0;
}

int GetFinalSimMatrix(int**             finalSimMatrix,
                      SimilarityMatrix& simMatrix,
                      float**           simGraphletMatrix,
                      float             coefSimGraphlet)
{

    for (int i = 0; i < simMatrix.nHNode; i++)
    {
        for (int j = 0; j < simMatrix.nGNode; j++)
        {
            finalSimMatrix[i][j] = (int)100*((1-coefSimGraphlet) * simMatrix.matrix[i][j] + coefSimGraphlet * simGraphletMatrix[i][j]);
        }
    }
    return 0;
}

int GetFinalSimMatrix(int**             finalSimMatrix,
                      SimilarityMatrix& oriSimMatrix,
                      SimilarityMatrix& neiSimMatrix,
                      float**           simGraphletMatrix)
{

    for (int i = 0; i < oriSimMatrix.nHNode; i++)
    {
        for (int j = 0; j < oriSimMatrix.nGNode; j++)
        {
            finalSimMatrix[i][j] = (int)100*(ORTH_COEF * oriSimMatrix.matrix[i][j] + (1 - GRAPHLET_COEF - ORTH_COEF) * neiSimMatrix.matrix[i][j] + GRAPHLET_COEF * simGraphletMatrix[i][j]);
        }
    }
    return 0;
}

int GetFinalSimMatrix(int**             finalSimMatrix,
                      SimilarityMatrix& simMatrix)
{
     for (int i = 0; i < simMatrix.nHNode; i++)
    {
        for (int j = 0; j < simMatrix.nGNode; j++)
        {
            finalSimMatrix[i][j] = (int)100*simMatrix.matrix[i][j];
        }
    }
    return 0;
}

int GetFinalSimMatrix(int                 **finalSimMatrix,
                      SimilarityMatrix&   simMatrix,
                      float               ***simDensNMatrix,
                      float               ***edgeDiffNMatrix,
                      float               fSimMatrixParam,
                      std::vector<float>& vSimDensParam,
                      std::vector<float>& vEdgeDiffParam,
                      int                 n)
{

    for (int i = 0; i < simMatrix.nHNode; i++)
    {
        for (int j = 0; j < simMatrix.nGNode; j++)
        {

            finalSimMatrix[i][j] = (int)100*fSimMatrixParam * simMatrix.matrix[i][j];
            for (int k = 0; k < n; k++)
            {
                finalSimMatrix[i][j] += (int)100*(vSimDensParam[k] * simDensNMatrix[k][i][j] + vEdgeDiffParam[k] * edgeDiffNMatrix[k][i][j]);
            }

        }

    }
    return 0;
}


int GetFixedVar(Network&          hNetwork,
                Network&          gNetwork,
                std::vector<int>* fixedVarIntList,
                const char*       alignmentFile)
{
    ifstream ifs(alignmentFile);
    if(!ifs.good())
    {
        cerr << "Input file error!" << endl;
        exit(-1);
    }

    vector<string> alignmentStr[2];
    string strTmp1, strTmp2, strTmp3;
    while(ifs >> strTmp1 >> strTmp2 >> strTmp3)
    {
        alignmentStr[0].push_back(strTmp1);
        alignmentStr[1].push_back(strTmp2);
        cout << strTmp1 << " " << strTmp2 << " " << strTmp3 << endl;
        strTmp1.clear();
        strTmp2.clear();
        strTmp3.clear();
    }

    for (int i = 0; i < alignmentStr[0].size(); i++)
    {
        fixedVarIntList[0].push_back(hNetwork.GetNodeIndex(alignmentStr[0][i]));
        fixedVarIntList[1].push_back(gNetwork.GetNodeIndex(alignmentStr[1][i]));
    }
    return 0;
}

int GetFixedVar(Network&             hNetwork,
                Network&             gNetwork,
                std::vector<string>* vAlignmentNodeStr,
                const char*          alignmentFile)
{
    ifstream ifs(alignmentFile);
    if(!ifs.good())
    {
        cerr << "Input file error!" << endl;
        exit(-1);
    }

    string strTmp1, strTmp2, strTmp3;
    while(ifs >> strTmp1 >> strTmp2 >> strTmp3)
    {
        vAlignmentNodeStr[0].push_back(strTmp1);
        vAlignmentNodeStr[1].push_back(strTmp2);
        cout << strTmp1 << " " << strTmp2 << " " << strTmp3 << endl;
        strTmp1.clear();
        strTmp2.clear();
        strTmp3.clear();
    }



    // print fixedVarIntList
    for (int i = 0; i < vAlignmentNodeStr[0].size(); i++)
    {
        cout << vAlignmentNodeStr[0][i] << " " << vAlignmentNodeStr[1][i] << endl;
    }
    return 0;
}

int RemoveNegContributionAlignedPairs(Network&          hNetwork,
                                      Network&          gNetwork,
                                      SimilarityMatrix& simMatrix,
                                      vector<string>*   vAlignmentNodeStr,
                                      vector<string>*   posContribAlignedPairs)
{
    vector<string> hAlignEdge[2];
    vector<string> gAlignEdge[2];
    vector<string> hgAlignEdge[2]; // the intersection of hAlignEdge and gAlignEdge

    for (int i = 0; i < hNetwork.nEdge; i++)
    {
        vector<string>::iterator itr1, itr2;

        itr1 = find(vAlignmentNodeStr[0].begin(), vAlignmentNodeStr[0].end(), hNetwork.netStrVec[0][i]);
        itr2 = find(vAlignmentNodeStr[0].begin(), vAlignmentNodeStr[0].end(), hNetwork.netStrVec[1][i]);
        if (itr1 != vAlignmentNodeStr[0].end() && itr2 != vAlignmentNodeStr[0].end())
        {
            string label1, label2;
            int index1, index2;

            index1 = itr1 - vAlignmentNodeStr[0].begin();
            index2 = itr2 - vAlignmentNodeStr[0].begin();

            label1 = hNetwork.netStrVec[0][i];
            label1 += "/";
            label1 += vAlignmentNodeStr[1][index1];

            label2 = hNetwork.netStrVec[1][i];
            label2 += "/";
            label2 += vAlignmentNodeStr[1][index2];

            hAlignEdge[0].push_back(label1);
            hAlignEdge[1].push_back(label2);
        }
    }

    for (int i = 0; i < gNetwork.nEdge; i++)
    {
        vector<string>::iterator itr1, itr2;

        itr1 = find(vAlignmentNodeStr[1].begin(), vAlignmentNodeStr[1].end(), gNetwork.netStrVec[0][i]);
        itr2 = find(vAlignmentNodeStr[1].begin(), vAlignmentNodeStr[1].end(), gNetwork.netStrVec[1][i]);
        if (itr1 != vAlignmentNodeStr[1].end() && itr2 != vAlignmentNodeStr[1].end())
        {
            string label1, label2;
            int index1, index2;

            index1 = itr1 - vAlignmentNodeStr[1].begin();
            index2 = itr2 - vAlignmentNodeStr[1].begin();

            label1 = vAlignmentNodeStr[0][index1];
            label1 += "/";
            label1 += gNetwork.netStrVec[0][i];

            label2 = vAlignmentNodeStr[0][index2];
            label2 += "/";
            label2 += gNetwork.netStrVec[1][i];

            gAlignEdge[0].push_back(label1);
            gAlignEdge[1].push_back(label2);
        }
    }

    for (int i = 0; i < hAlignEdge[0].size(); i++)
    {
        for (int j = 0; j < gAlignEdge[0].size(); j++)
        {
            if ((hAlignEdge[0][i] == gAlignEdge[0][j]  &&
                    hAlignEdge[1][i] == gAlignEdge[1][j]) ||
                    (hAlignEdge[0][i] == gAlignEdge[1][j]  &&
                    hAlignEdge[1][i] == gAlignEdge[0][j]))
            {
                hgAlignEdge[0].push_back(hAlignEdge[0][i]);
                hgAlignEdge[1].push_back(hAlignEdge[1][i]);
                break;
            }
        }
    }

    for (int i = 0; i < vAlignmentNodeStr[0].size(); i++)
    {
        float score = (GAMMA-DELTA)*simMatrix.matrix[hNetwork.GetNodeIndex(vAlignmentNodeStr[0][i])][gNetwork.GetNodeIndex(vAlignmentNodeStr[1][i])] + DELTA;
        string nodeStr;
        int nAlignedEdge = 0;
        nodeStr = vAlignmentNodeStr[0][i];
        nodeStr += "/";
        nodeStr += vAlignmentNodeStr[1][i];
        for (int j = 0; j < hgAlignEdge[0].size(); j++)
        {
            if (hgAlignEdge[0][j] == nodeStr || hgAlignEdge[1][j] == nodeStr)
            {
                nAlignedEdge += 1;
            }
        }
        cout << "nAlignedEdge " << nAlignedEdge << endl;
        score += (ALPHA-2*BETA) * nAlignedEdge;
        if (score > 0)
        {
            posContribAlignedPairs[0].push_back(vAlignmentNodeStr[0][i]);
            posContribAlignedPairs[1].push_back(vAlignmentNodeStr[1][i]);
        }
    }

    return 0;
}

void GenerateAlignmentFile(Network&           hNetwork,
                           Network&           gNetwork,
                           const char*        alignmentFile,
                           vector<string>*    vAlignmentNodeStr,
                           SimilarityMatrix&  simMatrix)
{
    ofstream aFP(alignmentFile);
    aFP << "node1" << "\t" << "node2" << "\t" << "ortholog" << endl;
    
    for (int i = 0; i < vAlignmentNodeStr[0].size(); i++)
    {
        unsigned int node1 = hNetwork.GetNodeIndex(vAlignmentNodeStr[0][i]);
        unsigned int node2 = gNetwork.GetNodeIndex(vAlignmentNodeStr[1][i]);
        double similarity = simMatrix.matrix[node1][node2];
        aFP << vAlignmentNodeStr[0][i] << "\t" << vAlignmentNodeStr[1][i] << "\t" << similarity << endl;
    }
    aFP.close();
}

int PrintAlignmentInfo(Network&          hNetwork,
                       Network&          gNetwork,
                       SimilarityMatrix& similarityMatrix,
                       vector<string>*   vAlignmentNodeStr)
{
    int orthCount = 0;
    int loopCount = 0;
    for (int i = 0; i < vAlignmentNodeStr[0].size(); i++)
    {
        int node1 = hNetwork.GetNodeIndex(vAlignmentNodeStr[0][i]);
        int node2 = gNetwork.GetNodeIndex(vAlignmentNodeStr[1][i]);
        if (similarityMatrix.matrix[node1][node2] > 0)
        {
            orthCount += 1;
        }
        if (hNetwork.isLoop[node1] && gNetwork.isLoop[node2])
        {
            loopCount += 1;
        }
    }

    int nAlignEdge = 0;
    for (int i = 0; i < hNetwork.netIntVec[0].size(); i++)
    {
        for (int j = 0; j < gNetwork.netIntVec[0].size(); j++)
        {
            bool align00 = false;
            bool align01 = false;
            bool align10 = false;
            bool align11 = false;

            for (int k = 0; k < vAlignmentNodeStr[0].size(); k++)
            {
                int node0 = hNetwork.GetNodeIndex(vAlignmentNodeStr[0][k]);
                int node1 = gNetwork.GetNodeIndex(vAlignmentNodeStr[1][k]);
                if (hNetwork.netIntVec[0][i] == node0 && gNetwork.netIntVec[0][j] == node1)
                {
                    align00=true;
                }
                if (hNetwork.netIntVec[0][i] == node0 && gNetwork.netIntVec[1][j] == node1)
                {
                    align01=true;
                }
                if (hNetwork.netIntVec[1][i] == node0 && gNetwork.netIntVec[0][j] == node1)
                {
                    align10=true;
                }
                if (hNetwork.netIntVec[1][i] == node0 && gNetwork.netIntVec[1][j] == node1)
                {
                    align11=true;
                }
                if (align00==true && align11==true)
                {
                    nAlignEdge += 1;
                    break;
                }
                else if (align01==true && align10==true)
                {
                    nAlignEdge += 1;
                    break;
                }
            }
        }
    }
    cout << "No. of aligned nodes : " << vAlignmentNodeStr[0].size() << endl;
    cout << "No. of aligned orths : " << orthCount << endl;
    cout << "No. of aligned edges : " << nAlignEdge << endl;
    cout << "No. of aligned loops : " << loopCount << endl;
    return 0;
}
