/*
 * graph.cpp
 *
 * Copyright 2011-2012. All rights reserved.
 */

#include "graph.h"

using namespace std;

int GenerateDotFile(Network&    network,
                    const char* netGraphFile)
{
    ofstream ofs(netGraphFile);

    ofs << "Graph G {\n";
    for(int itr = 0; itr != network.nEdge; itr++)
    {
        ofs << "\"" << network.netStrVec[0][itr] << "\"--\"" << network.netStrVec[1][itr] << "\";" << endl;
    }
    ofs << "}\n";

    ofs.close();
    return 0;
}

int GenerateAlignmentDotFile(Network&          hNetwork,
                             Network&          gNetwork,
                             const char*       alignGraphFile,
                             vector<string>*   vAlignmentNodeStr,
                             SimilarityMatrix& simMatrix)
{
    ofstream ofs(alignGraphFile);

    vector<string> hAlignEdge[2];
    vector<string> gAlignEdge[2];
    vector<string> hgAlignEdge[2]; // the intersection of hAlignEdge and gAlignEdge
    
    // print comments
    ofs << "/*" << "\n" 
        << " * LINKS:" << "\n"
        << " * blue  := links only in network 1" << "\n"
        << " * green := links only in network 2" << "\n"
        << " * red   := links in both network 1 and network 2" << "\n"
        << " *" << "\n"
        << " * NODES:" << "\n"
        << " * pink  := aligned ortholog pairs" << "\n"
        << " */" << "\n"
        << endl;
    
    ofs << "Graph G {\n";
    for (int i = 0; i < hNetwork.nEdge; i++)
    {
        vector<string>::iterator itr1, itr2;

        itr1 = find(vAlignmentNodeStr[0].begin(), vAlignmentNodeStr[0].end(), hNetwork.netStrVec[0][i]);
        itr2 = find(vAlignmentNodeStr[0].begin(), vAlignmentNodeStr[0].end(), hNetwork.netStrVec[1][i]);
        if (itr1 != vAlignmentNodeStr[0].end() && itr2 != vAlignmentNodeStr[0].end())
        {
            string label1, label2;
            int index1, index2;

            index1 = itr1 - vAlignmentNodeStr[0].begin();
            index2 = itr2 - vAlignmentNodeStr[0].begin();

            label1 = hNetwork.netStrVec[0][i];
            label1 += "/";
            label1 += vAlignmentNodeStr[1][index1];

            label2 = hNetwork.netStrVec[1][i];
            label2 += "/";
            label2 += vAlignmentNodeStr[1][index2];

            hAlignEdge[0].push_back(label1);
            hAlignEdge[1].push_back(label2);
        }
    }

    for (int i = 0; i < gNetwork.nEdge; i++)
    {
        vector<string>::iterator itr1, itr2;

        itr1 = find(vAlignmentNodeStr[1].begin(), vAlignmentNodeStr[1].end(), gNetwork.netStrVec[0][i]);
        itr2 = find(vAlignmentNodeStr[1].begin(), vAlignmentNodeStr[1].end(), gNetwork.netStrVec[1][i]);
        if (itr1 != vAlignmentNodeStr[1].end() && itr2 != vAlignmentNodeStr[1].end())
        {
            string label1, label2;
            int index1, index2;

            index1 = itr1 - vAlignmentNodeStr[1].begin();
            index2 = itr2 - vAlignmentNodeStr[1].begin();

            label1 = vAlignmentNodeStr[0][index1];
            label1 += "/";
            label1 += gNetwork.netStrVec[0][i];

            label2 = vAlignmentNodeStr[0][index2];
            label2 += "/";
            label2 += gNetwork.netStrVec[1][i];

            gAlignEdge[0].push_back(label1);
            gAlignEdge[1].push_back(label2);
        }
    }


    char hLabelMark[hAlignEdge[0].size()];
    char gLabelMark[gAlignEdge[0].size()];

    //initialize the labelMark;
    for (int i = 0; i < hAlignEdge[0].size(); i++) hLabelMark[i] = 'n';
    for (int i = 0; i < gAlignEdge[0].size(); i++) gLabelMark[i] = 'n';

    for (int i = 0; i < hAlignEdge[0].size(); i++)
    {
        for (int j = 0; j < gAlignEdge[0].size(); j++)
        {
            if ((hAlignEdge[0][i] == gAlignEdge[0][j]  &&
                    hAlignEdge[1][i] == gAlignEdge[1][j]) ||
                    (hAlignEdge[0][i] == gAlignEdge[1][j]  &&
                    hAlignEdge[1][i] == gAlignEdge[0][j]))
            {
                hgAlignEdge[0].push_back(hAlignEdge[0][i]);
                hgAlignEdge[1].push_back(hAlignEdge[1][i]);
                hLabelMark[i] = 'y';
                gLabelMark[j] = 'y';
                break;
            }
        }

    }
    for (int i = 0; i < hAlignEdge[0].size(); i++)
    {
        if (hLabelMark[i] == 'n')
        {
            ofs << "{\"" << hAlignEdge[0][i] << "\"--\"" << hAlignEdge[1][i] << "\"[color=blue]};" << endl;
        }
    }

    ofs << "\n\n\n";

    for (int i = 0; i < gAlignEdge[0].size(); i++)
    {
        if (gLabelMark[i] == 'n')
        {
            ofs << "{\"" << gAlignEdge[0][i] << "\"--\"" << gAlignEdge[1][i] << "\"[color=green]};" << endl;
        }
    }

    ofs << "\n\n\n";

    for (int i = 0; i < hgAlignEdge[0].size(); i++)
    {
        ofs << "{\"" << hgAlignEdge[0][i] << "\"--\"" << hgAlignEdge[1][i] << "\"[color=red]};" << endl;
    }

    ofs << "\n\n\n";

    for (int i = 0; i < vAlignmentNodeStr[0].size(); i++)
    {
        int hNode = hNetwork.GetNodeIndex(vAlignmentNodeStr[0][i]);
        int gNode = gNetwork.GetNodeIndex(vAlignmentNodeStr[1][i]);

        string labelTmp;
        labelTmp += vAlignmentNodeStr[0][i];
        labelTmp += "/";
        labelTmp += vAlignmentNodeStr[1][i];

        if (hNetwork.isNonIsolated[hNode] == false &&
            gNetwork.isNonIsolated[gNode] == false)
        {
            if (hNetwork.isLoop[hNode] == false && gNetwork.isLoop[gNode] == false)
            {
                ofs << "{\"" << labelTmp << "\"};" << endl;
            }
            else if (hNetwork.isLoop[hNode] == true && gNetwork.isLoop[gNode] == false)
            {
                ofs << "{\"" << labelTmp << "\"--\"" << labelTmp << "\"[color=blue]};" << endl;
            }
            else if (hNetwork.isLoop[hNode] == false && gNetwork.isLoop[gNode] == true)
            {
                ofs << "{\"" << labelTmp << "\"--\"" << labelTmp << "\"[color=green]};" << endl;
            }
            else
            {
                ofs << "{\"" << labelTmp << "\"--\"" << labelTmp << "\"[color=red]};" << endl;
            }
        }
        else
        {
            if (hNetwork.isLoop[hNode] == true && gNetwork.isLoop[gNode] == false)
            {
                ofs << "{\"" << labelTmp << "\"--\"" << labelTmp << "\"[color=blue]};" << endl;
            }
            else if (hNetwork.isLoop[hNode] == false && gNetwork.isLoop[gNode] == true)
            {
                ofs << "{\"" << labelTmp << "\"--\"" << labelTmp << "\"[color=green]};" << endl;
            }
            else if (hNetwork.isLoop[hNode] == true && gNetwork.isLoop[gNode] == true)
            {
                ofs << "{\"" << labelTmp << "\"--\"" << labelTmp << "\"[color=red]};" << endl;
            }
        }

        if (simMatrix.matrix[hNode][gNode] > 0)
        {
            ofs << "{\"" << labelTmp << "\" [color=lightpink, style=filled]};" << endl;
        }
    }

    ofs << "}\n";
    ofs.close();

    return 0;
}
