/*
 * network.cpp
 *
 * Copyright 2011-2012. All rights reserved.
 */

#include <cstdlib>
#include "network.h"

using namespace std;

Network::Network() :
nNode(0),
nEdge(0),
nLoop(0),
nIsolated(0),
nLinkedNode(0),
netMatrix(NULL)
{
}


Network::~Network()
{
    for(int i = 0; i < nLinkedNode; i++)
    {
        delete[] netMatrix[i];
        netMatrix[i] = NULL;
    }
    delete[] netMatrix;
    netMatrix = NULL;
}


int Network::PrintIsolatedNodeIntVec()
{
    for(int i = 0; i < isolatedNodeIntVec.size(); i++)
    {
        cout << isolatedNodeIntVec[i] << endl;
    }
    return 0;
}

int Network::PrintNetInfo()
{
    cout << "The No. of nodes:           " << nNode << endl;
    cout << "The No. of isolated nodes:  " << nIsolated << endl;
    cout << "The No. of edges:           " << nEdge << endl;
    cout << "The No. of loops:           " << nLoop << endl;
    return 0;
}

int Network::PrintNetMatrix()
{
    cout << "net matrix: " << endl;
    for (int i = 0; i < nLinkedNode; i++)
    {
        for (int j = 0; j < nLinkedNode; j++)
        {
            cout << netMatrix[i][j];
            if (j < nNode - 1) cout << " ";
        }
        cout << endl;
    }
    return 0;
}

int Network::PrintNetStrVec()
{
    cout << "P_A" << "\t" << "P_B" << endl;
    for (size_t i = 0; i < netStrVec[0].size(); i++)
    {
        cout << netStrVec[0][i] << "\t" << netStrVec[1][i] << endl;
    }
    return 0;
}

int Network::PrintNetIntVec()
{
    cout << "P_A" << "\t" << "P_B" << endl;
    for (size_t i = 0; i < netIntVec[0].size(); i++)
    {
        cout << netIntVec[0][i] << "\t" << netIntVec[1][i] << endl;
    }
    return 0;
}

int Network::PrintNodeStrList()
{
    int index = 0;
    cout << "index\tlabel" << endl;
    for (list<string>::iterator itr = nodeStrList.begin(); itr != nodeStrList.end(); itr++)
    {
        cout << index++ << "\t" << *itr << endl;
    }
    return 0;
}

int Network::PrintNodeStrVec()
{
    cout << "index\tlabel" << endl;
    for (int i = 0; i < nodeStrVec.size(); i++)
    {
        cout << i << "\t" << nodeStrVec[i] << endl;
    }
    return 0;
}

int Network::PrintNodeMap()
{
    cout << "node str. to index map: " << endl;
    for (map<string, int>::iterator itr = nodeMap.begin(); itr != nodeMap.end(); itr++)
    {
        cout << itr->first << " --> " << itr->second << endl;
    }

    return 0;
}

int Network::GetNetNMatrices(int*** netNMatrix, int n)
{
    if (n <= 0)
    {
        cout << "n should be larger than 1!" << endl;
        exit(1);
    }


    for (int i = 0; i < n; i++)
    {
        netNMatrix[i] = new int* [nNode];
        for (int j = 0; j < nNode; j++)
        {
            netNMatrix[i][j] = new int[nNode];
        }
        if (i == 0)
        {
            GetNetNthMatrix(netMatrix, netMatrix, netNMatrix[i]);
        }
        else if (i == 1)
        {
            GetNetNthMatrix(netMatrix, netNMatrix[i-1], netNMatrix[i]);
        }
        else
        {
            GetNetNthMatrix(netNMatrix[i-2], netNMatrix[i-1], netNMatrix[i]);
        }
    }
    return 0;
}

int Network::GetNetNthMatrix(int **netN1Matrix, int **netN2Matrix, int **netN3Matrix)
{
    //M(N+2) = M(N)*M(N+1)+M(N+1)
    int minEntry = 1; // excluding the diagnal elments
    for (int i = 0; i < nNode; i++)
    {
        for (int j = 0; j < nNode; j++)
        {
            netN3Matrix[i][j] = 0;
            for (int k = 0; k < nNode; k++)
            {
                netN3Matrix[i][j] += netN1Matrix[i][k]*netN2Matrix[k][j];
            }
            netN3Matrix[i][j] += netN2Matrix[i][j];
            if (i != j)
            {
                minEntry = minEntry < netN3Matrix[i][j] ? minEntry : netN3Matrix[i][j];
            }
        }
    }
    cout << "minEntry : " << minEntry << endl;

    return 0;
}

int Network::CleanNetNMatrices(int ***netNMatrix, int n)
{
    for (int i = 0; i < n ; i++)
    {
        for (int j = 0; j < nNode; j++)
        {
            delete[] netNMatrix[i][j];
            netNMatrix[i][j] = NULL;
        }
        delete[] netNMatrix[i];
        netNMatrix[i] = NULL;
    }
    return 0;
}

int Network::PrintNetNMatrices(int ***netNMatrix, int n)
{
    for (int i = 0; i < n; i++)
    {
        cout << "matrix : " << i << endl;
        for (int j = 0; j < nNode; j++)
        {
            for (int k = 0; k < nNode; k++)
            {
                cout << netNMatrix[i][j][k] << " ";
            }
            cout << endl;
        }
        cout << "-----------------------------------" << endl;
    }
    return 0;
}



int Network::GetSubNetwork(int nodeIndex, SubNetwork& subNetwork) //nodeIndex is the index in Network
{
    subNetwork.centerNode = linkedNodeIntVec[nodeIndex];
    int i, j;
    //add node with label nodeNum to the subNetwork
    subNetwork.nNode = 0; //the node labeled by nodeNum
    subNetwork.nEdge = 0;
    //assume that no loop in the network

    subNetwork.nNode = netIntNeighborVec[nodeIndex].size() + 1;

    subNetwork.nodeIntVec = netIntNeighborVec[nodeIndex];
    for (i = 0; i < subNetwork.nodeIntVec.size(); i++)
    {
        subNetwork.nodeMap[subNetwork.nodeIntVec[i]] = i;
        subNetwork.isLoop.push_back(isLoop[subNetwork.nodeIntVec[i]]);
        subNetwork.nLoop += isLoop[subNetwork.nodeIntVec[i]];
    }

    subNetwork.isCenterNodeLoop = isLoop[subNetwork.centerNode];
    subNetwork.nEdge += subNetwork.nNode - 1;


    // find all the links between non-fixed node (nodeNum);
    for (i = 0; i < subNetwork.nNode - 2; i++)
    {
        for (j = i + 1; j < subNetwork.nNode - 1; j++)
        {
            if (netMatrix[subNetwork.nodeIntVec[i]][subNetwork.nodeIntVec[j]] == 1)
            {
                subNetwork.nEdge += 1;
                subNetwork.netIntVec[0].push_back(i);
                subNetwork.netIntVec[1].push_back(j);
            }
        }
    }

    //initialize netMatrix of subNetwork

    subNetwork.netMatrix = new int* [subNetwork.nNode - 1];
    for (i = 0; i < subNetwork.nNode - 1; i++)
    {
        subNetwork.netMatrix[i] = new int[subNetwork.nNode - 1];
        for (j = 0; j < subNetwork.nNode - 1; j++)
        {
            subNetwork.netMatrix[i][j] = 0;
        }
    }

    for (i = 0; i < subNetwork.nEdge - subNetwork.nNode + 1; i++)
    {
        subNetwork.netMatrix[subNetwork.netIntVec[0][i]][subNetwork.netIntVec[1][i]] = 1;
        subNetwork.netMatrix[subNetwork.netIntVec[1][i]][subNetwork.netIntVec[0][i]] = 1;
    }

    //Initialize nodeStrVec

    for (i = 0; i < subNetwork.nNode - 1; i++)
    {
        subNetwork.nodeStrVec.push_back(nodeStrVec[linkedNodeIntVec[subNetwork.nodeIntVec[i]]]);
    }
    //subNetwork.PrintNodeStrVec();

    return 0;
}

int Network::GetSubNetworks(SubNetwork* subNetworks)
{
    for (int i = 0; i < nLinkedNode; i++)
    {
        GetSubNetwork(i, subNetworks[i]);
    }
    return 0;
}

int Network::PrintLinkedNodeVec()
{
    cout << "linked node int. list: " << endl;
    cout << "index\tlabel" << endl;
    for (int i = 0; i < linkedNodeIntVec.size(); i++)
    {
        cout << i << "\t" << linkedNodeIntVec[i] << endl;
    }
    return 0;
}

int Network::PrintStr2IntLinkedNodeMap()
{
    cout << "str. to linked node index: " << endl;
    for (map<string, int>::iterator itr = str2IntLinkedNodeMap.begin(); itr != str2IntLinkedNodeMap.end(); itr++)
    {
        cout << "--" << itr->first << "-->" << itr->second << endl;
    }
    return 0;
}
int Network::PrintIsolatedNodeVec()
{
    cout << "isolated node int. list: " << endl;
    cout << "index\tlabel" << endl;
    for (int i = 0; i < isNonIsolated.size(); i++)
    {
        if (isNonIsolated[i] == false)
        {
            cout << i << "\t" << nodeStrVec[i] << endl;
        }
    }
    return 0;
}

int Network::PrintLoopNodeVec()
{
    cout << "loop node int. list: " << endl;
    cout << "index\tlabel" << endl;
    for (int i = 0; i < isLoop.size(); i++)
    {
        if (isLoop[i] == true)
        {
            cout << i << "\t" << nodeStrVec[i] << endl;
        }
    }
}

int Network::InitializeNetwork(const char* nodeFile, const char* netFile)
{
    int i, j, k;
    //initialize the string node list
    string strTmp1, strTmp2;
    ifstream ifs1,ifs2;
    ifs1.open(nodeFile);
    if(!ifs1.good())
    {
        cerr << "Input file error*!" << endl;
        exit(-1);
    }
    while(ifs1 >> strTmp1)
    {
        nodeStrList.push_back(strTmp1);
        strTmp1.clear();
    }
    ifs1.close();

    nodeStrList.sort();
    nodeStrList.unique();
    nNode = nodeStrList.size();

    //initialize the map

    i = 0;
    for (list<string>::iterator itr = nodeStrList.begin(); itr != nodeStrList.end(); itr++)
    {
        nodeStrVec.push_back(*itr);
        nodeMap[*itr] = i++;
        isLoop.push_back(false);
        isNonIsolated.push_back(false);
    }

    //initialize str. net list

    list<string> netStrList;

    ifs2.open(netFile);

    if(!ifs2.good())
    {
        cerr << "Input file error+!" << endl;
        exit(-1);
    }


    while(ifs2 >> strTmp1 >> strTmp2)
    {
        string strTmp;
        if (strTmp1 > strTmp2)
        {
            strTmp += strTmp2;
            strTmp += ' ';
            strTmp += strTmp1;
        }
        else
        {
            strTmp += strTmp1;
            strTmp += ' ';
            strTmp += strTmp2;
        }
        netStrList.push_back(strTmp);
        strTmp1.clear();
        strTmp2.clear();
    }
    ifs2.close();

    netStrList.sort();
    netStrList.unique();

    for (list<string>::iterator itr = netStrList.begin(); itr != netStrList.end(); itr++)
    {
        stringstream ss;
        ss << *itr;
        ss >> strTmp1 >> strTmp2;
        if (strTmp1 == strTmp2)
        {
            isLoop[GetNodeIndex(strTmp1)] = true;
            nLoop += 1;
        }
        else
        {
            netStrVec[0].push_back(strTmp1);
            netStrVec[1].push_back(strTmp2);
            nLinkedNode += 1-isNonIsolated[GetNodeIndex(strTmp1)];
            nLinkedNode += 1-isNonIsolated[GetNodeIndex(strTmp2)];
            isNonIsolated[GetNodeIndex(strTmp1)] = true;
            isNonIsolated[GetNodeIndex(strTmp2)] = true;
            nEdge += 1;
        }
        strTmp1.clear();
        strTmp2.clear();
    }

    //initialize isolatedNodeIntVec, linkedNodeIntVec, str2IntLinkedNodeMap, str2IntIsoNodeMap
    j=0;
    k=0;
    for(i = 0; i < nNode; i++)
    {
        if (isNonIsolated[i] == true)
        {
            linkedNodeIntVec.push_back(i);
            str2IntLinkedNodeMap[nodeStrVec[i]] = j;
            j++;
        }
        else
        {
            isolatedNodeIntVec.push_back(i);
            str2IntIsoNodeMap[nodeStrVec[i]] = k;
            k++;
        }
    }

    //initialize int. net list
    for (i = 0; i < nEdge; i++)
    {
        netIntVec[0].push_back(nodeMap.find(netStrVec[0][i])->second);
        netIntVec[1].push_back(nodeMap.find(netStrVec[1][i])->second);
    }
    nIsolated = nNode - nLinkedNode;

    //initialize netMatrix

    netMatrix = new int* [nLinkedNode];
    for (i = 0; i < nLinkedNode; i++)
    {
        netMatrix[i] = new int[nLinkedNode];
        for (j = 0; j < nLinkedNode; j++)
        {
            netMatrix[i][j] = 0;
        }
    }

    for (i = 0; i < nEdge; i++)
    {
        int node1 = GetLinkedNodeIndex(netIntVec[0][i]);
        int node2 = GetLinkedNodeIndex(netIntVec[1][i]);
        netMatrix[node1][node2] = 1;
        netMatrix[node2][node1] = 1;
    }

    //initialize netIntNeighborList

    for (i = 0; i < nLinkedNode; i++)
    {
        vector<int> tmp;
        netIntNeighborVec.push_back(tmp);
        for (j = 0; j < nLinkedNode; j++)
        {
            if (netMatrix[i][j] == 1)
            {
                netIntNeighborVec[i].push_back(j); //this variable just stores the index in linked node set, not in whole network node set
            }
        }
    }



    return 0;
}

int Network::InitializeNetwork(const char* netFileName)
{
    int i, j;
    //initialize the string net list and string node list
    ifstream ifs(netFileName);
    if(!ifs.good())
    {
        cerr << "Input file error!" << endl;
        exit(-1);
    }
    string strTmp1, strTmp2;
    while(ifs >> strTmp1 >> strTmp2)
    {
        netStrVec[0].push_back(strTmp1);
        netStrVec[1].push_back(strTmp2);
        nodeStrVec.push_back(strTmp1);
        nodeStrVec.push_back(strTmp2);
        nEdge += 1;
        strTmp1.clear();
        strTmp2.clear();
    }

    ifs.close();

    nodeStrList.sort();
    nodeStrList.unique();

    nNode = nodeStrList.size();

    //initialize the map
    i = 0;
    for (list<string>::iterator itr = nodeStrList.begin(); itr != nodeStrList.end(); itr++)
    {
        nodeStrVec.push_back(*itr);
        nodeMap[*itr] = i++;
    }
    //initialize int net list
    for (i = 0; i < nEdge; i++)
    {
        netIntVec[0].push_back(nodeMap.find(netStrVec[0][i])->second);
        netIntVec[1].push_back(nodeMap.find(netStrVec[1][i])->second);
    }

    //initialize netMatrix

    netMatrix = new int* [nNode];
    for (i = 0; i < nNode; i++)
    {
        netMatrix[i] = new int[nNode];
        for (j = 0; j < nNode; j++)
        {
            netMatrix[i][j] = 0;
        }
    }

    for (i = 0; i < nEdge; i++)
    {
        netMatrix[netIntVec[0][i]][netIntVec[1][i]] = 1;
        netMatrix[netIntVec[1][i]][netIntVec[0][i]] = 1;
    }

    //initialize netIntNeighborList

    for (i = 0; i < nNode; i++)
    {
        vector<int> tmp;
        netIntNeighborVec.push_back(tmp);
        for (j = 0; j < nNode; j++)
        {
            if (netMatrix[i][j] == 1)
            {
                netIntNeighborVec[i].push_back(j);
            }
        }
    }

    return 0;
}


int Network::PrintNetIntNeighborVec()
{
    int i, j;
    for (i = 0; i < nLinkedNode; i++)
    {
        cout << i << " :";
        for (j = 0; j < netIntNeighborVec[i].size(); j++)
        {
            cout << " " << netIntNeighborVec[i][j];
        }
        if (j == 0)
        {
            cout << "NULL;";
        }
        cout << endl;
    }
    return 0;
}

int Network::GetNodeIndex(string& nodeLabel)
{
    int nodeIndex;
    map<string, int>::iterator itr=nodeMap.find(nodeLabel);
    if (itr == nodeMap.end())
    {
        cerr << "ERROR: cannot find \"" << nodeLabel << "\" in node name list." << endl;
        exit(-1);
    }

    nodeIndex = itr->second;
    return nodeIndex;
}

int Network::GetLinkedNodeIndex(string& nodeLabel)
{
    int linkedNodeIndex;
    map<string, int>::iterator itr = str2IntLinkedNodeMap.find(nodeLabel);
    if (itr == str2IntLinkedNodeMap.end())
    {
        cerr << "ERROR: cannot find \"" << nodeLabel << "\" in str2IntLinkedNodeMap." << endl;
        exit(-1);
    }
    linkedNodeIndex = itr->second;
    return linkedNodeIndex;
}
int Network::GetLinkedNodeIndex(int nodeIndex)
{
    int linkedNodeIndex;
    map<string, int>::iterator itr = str2IntLinkedNodeMap.find(nodeStrVec[nodeIndex]);
    if (itr == str2IntLinkedNodeMap.end())
    {
        cerr << "ERROR: cannot find \"" << nodeStrVec[nodeIndex] << "\" in str2IntLinkedNodeMap." << endl;
        exit(-1);
    }
    linkedNodeIndex = itr->second;
    return linkedNodeIndex;
}

int Network::GetIsoNodeIndex(int nodeIndex)
{
    int isoNodeIndex;
    map<string, int>::iterator itr = str2IntIsoNodeMap.find(nodeStrVec[nodeIndex]);
    if (itr == str2IntIsoNodeMap.end())
    {
        cerr << "ERROR: cannot find \"" << nodeStrVec[nodeIndex] << "\" in str2IntIsoNodeMap." << endl;
        exit(-1);
    }
    isoNodeIndex = itr->second;
    return isoNodeIndex;
}

int Network::GetIsoNodeIndex(string& nodeLabel)
{
    int isoNodeIndex;
    map<string, int>::iterator itr = str2IntIsoNodeMap.find(nodeLabel);
    if (itr == str2IntIsoNodeMap.end())
    {
        cerr << "ERROR: cannot find \"" << nodeLabel << "\" in str2IntIsoNodeMap." << endl;
        exit(-1);
    }
    isoNodeIndex = itr->second;
    return isoNodeIndex;
}
