/*
 * network.h
 *
 * Copyright 2011-2012. All rights reserved.
 */

#ifndef JL_NETWORK_H_
#define JL_NETWORK_H_ 1

#include<iostream>
#include<fstream>
#include<string>
#include<list>
#include<vector>
#include<map>
#include<sstream>

#include "subnetwork.h"

class Network
{
public:    
    Network();
    ~Network();
    int InitializeNetwork(const char*);
    int InitializeNetwork(const char* hNodeFile,
                          const char* hNetFile);
    int GetSubNetwork(int         nodeNum,
                      SubNetwork& subNetwork);
    int GetSubNetworks(SubNetwork*);
    int GetNodeIndex(std::string& nodeLabel);
    int GetNodeIndex(int);
    int GetLinkedNodeIndex(std::string& nodeLabel);
    int GetLinkedNodeIndex(int nodeIndex);
    int GetIsoNodeIndex(std::string& nodeLabel);
    int GetIsoNodeIndex(int nodeIndex);
    int GetNetNMatrices(int*** netNMatrix,
                        int    n);
    int GetNetNthMatrix(int** netN1Matrix,
                        int** netN2Matrix,
                        int** netN3Matrix);
    int CleanNetNMatrices(int*** netNMatrix,
                          int    n);

    int PrintNetNMatrices(int*** netNMatrix,
                          int    n);
    int PrintNetMatrix();
    int PrintNetStrVec();
    int PrintNodeStrVec();
    int PrintNetIntVec();
    int PrintNodeStrList();
    int PrintNodeMap();
    int PrintNetIntNeighborVec();
    int PrintStr2IntLinkedNodeMap();
    int PrintNetInfo();
    int PrintLinkedNodeVec();
    int PrintIsolatedNodeVec();
    int PrintLoopNodeVec();
    int PrintIsolatedNodeIntVec();

    int nNode;
    int nEdge;
    int nIsolated;
    int nLoop;
    int nLinkedNode;
    int **netMatrix; //nLinkedNode*nLinkedNode
    std::vector<std::string> netStrVec[2];
    std::vector<std::string> nodeStrVec;
    std::vector<int> netIntVec[2];
    std::vector<int> linkedNodeIntVec;
    std::vector<int> isolatedNodeIntVec;
    std::vector< std::vector<int> > netIntNeighborVec; //this variable just stores the index in linked node set, not in whole network node set
    std::vector<bool> isLoop;
    std::vector<bool> isNonIsolated;// if a vectex just only has one loop and no other vertices linked to it, then it's also an isolated vertex.


private:
    std::list<std::string> nodeStrList;
    std::map<std::string, int> nodeMap;
    std::map<std::string, int> str2IntLinkedNodeMap;
    std::map<std::string, int> str2IntIsoNodeMap;
};

#endif // JL_NETWORK_H_
