/*
 * similaritymatrix.h
 *
 * Copyright 2011-2012. All rights reserved.
 */


#ifndef JL_SIMILARITYMATRIX_H_
#define JL_SIMILARITYMATRIX_H_ 1

#include<iostream>
#include<cmath>
#include<math.h>
#include "network.h"
#include "graphlet.h"

extern const int NPATTERN;



class SimilarityMatrix
{
public:

    SimilarityMatrix();
    ~SimilarityMatrix();

    int InitializeSimMatrix(Network&,
                            Network&,
                            const char*);
    int InitializeEmptySimMatrix(Network&,
                                 Network&);
    int InitializeBinarySimMatrix(Network& hNetwork,
                                  Network& gNetwork,
                                  const char* sFile);
    int CopySimMatrix(SimilarityMatrix& simMatrix);
    int MatrixNormalization();
    int MatrixNormalization(float** matrix,
                            float   max,
                            float   min,
                            float   maxSimNeiSimMatrix,
                            float   minSimNeiSimMatrix);
    int MatrixNormalization(float** matrix,
                            float   max);
    int PrintMatrix();
    int PrintSimilarityInfo();
    int UpdateMaxMinSim(float);
    int GetEdgeDiffNMatrices(float*** edgeDiffNMatrix,
                             int***   hNetNMatrix,
                             int***   gNetNMatrix,
                             Network& hNetwork,
                             Network& gNetwork,
                             int n);
    int GetSimGraphletMatrix(float**& simGraphletMatrix,
                             Network& hNetwork,
                             Network& gNetwork,
                             float&   maxSimGraphlet,
                             float&   minSimGraphlet);
    float GetPatternSim(int nNode,
                        int nEdge,
                        int orthCount,
                        int loopCount);
    int CleanSimDensNMatrices(float ***simDensNMatrix,
                              int n);
    int CleanEdgeDiffNMatrices(float*** edgeDiffNMatrix,
                               int      n);
    int CleanFinalSimMatrix(int** finalSimMatrix);
    int CleanSimGraphletMatrix(float** simGraphletMatrix);
    int PrintEdgeDiffNMatrices(float*** edgeDiffNMatrix,
                               int      n);
    int PrintSimDensNMatrices(float*** simDensNMatrix,
                              int      n);
    int PrintSimGraphletMatrix(float** simGraphletMatrix);
    int PrintSimGraphletMatrix(int**   simGraphletMatrix);
    int PrintUnalignedNodePairs();

    int nHNode;
    int nGNode;
    int nSim;
    float maxSim;
    float minSim;
    float **matrix;
    std::vector<int> unalignedNodePairs[2];

private:
    std::vector<std::string> simStrList[2];
    std::vector<float> similarityVec;
};


#endif // JL_SIMILARITYMATRIX_H_
